﻿/**
 * @file
 * @brief ResTex.hppの実装を記述する。
 * @author akino
 */
#include <XG3D/ResTex.hpp>

//------------------------------------------------------------
#include <XBase/EnumCheck.hpp>
#include <XBase/RuntimeAssert.hpp>
#include <XG3D/Engine.hpp>
#include "BinResTex.hpp"

//------------------------------------------------------------
namespace XG3D {
//------------------------------------------------------------
ResTex::ResTex()
: mPtr( 0 )
{
}

//------------------------------------------------------------
ResTex::ResTex( const const_ptr_t aBin )
: mPtr( 0 )
{
    // XData作成
    ::XData::XData xdata( aBin );
    if ( !xdata.isValidData() )
    {// 無効なデータ
        return;
    }

    // バイナリチェック
    const BinResTex* ptr = static_cast< const BinResTex* >( xdata.dataHeadAddress() );
    if ( ptr->header.checkInvalid( BinResType_Tex ) )
    {
        return;
    }

    // 正当なデータと判断
    mPtr = aBin;
}

//------------------------------------------------------------
bool ResTex::isValid()const
{
    return mPtr != 0;
}

//------------------------------------------------------------
u16 ResTex::width()const
{
    // チェック
    if ( checkInvalid() )
    {
        return u16();
    }
    return bin().width;
}

//------------------------------------------------------------
u16 ResTex::height()const
{
    // チェック
    if ( checkInvalid() )
    {
        return u16();
    }
    return bin().height;
}

//------------------------------------------------------------
ResTexFormat ResTex::format()const
{
    // チェック
    if ( checkInvalid() )
    {
        return ResTexFormat_MIN;
    }
    ResTexFormat fmt = ResTexFormat( bin().format );
    if ( XBASE_ENUM_IS_INVALID( ResTexFormat , fmt ) )
    {
        XBASE_NOT_REACH_ASSERT();
        return ResTexFormat_MIN;
    }
    return ResTexFormat( fmt );
}

//------------------------------------------------------------
const byte_t* ResTex::pixels()const
{
    // チェック
    if ( checkInvalid() )
    {
        return 0;
    }
    return static_cast< const byte_t* >( ref( bin().pixels ) );
}

//------------------------------------------------------------
bool ResTex::checkInvalid()const
{
    // 正しければ何もしない
    if ( isValid() )
    {
        return false;
    }

    // 不正なのでエラー
    XBASE_NOT_REACH_ASSERT();
    return true;
}

//------------------------------------------------------------
const BinResTex& ResTex::bin()const
{
    XBASE_ASSERT( isValid() );
    return *static_cast< const BinResTex* >( ::XData::XData( mPtr ).dataHeadAddress() );
}

//------------------------------------------------------------
const_ptr_t ResTex::ref( const ::XData::Reference aVal )const
{
    // ポインタ取得
    XBASE_ASSERT( isValid() );
    const void* ptr = ::XData::XData( mPtr ).labelAddressWithReference( aVal );

    // エラーチェック
    if ( ::XBase::PointerCheck::InvalidCheck( ptr ) )
    {
        return 0;
    }

    // 結果を返す
    return const_ptr_t( ptr );
}

//------------------------------------------------------------
}
//------------------------------------------------------------
// EOF
