﻿/**
 * @file
 * @brief TexBuffer.hppの実装を記述する。
 * @author akino
 */
#include <XG3D/TexBuffer.hpp>

//------------------------------------------------------------
#include <XBase/RuntimeAssert.hpp>
#include <XG3D/SDKHeader.hpp>
#include <XG3D/TexSetting.hpp>
#include "GLCMD.hpp"

//------------------------------------------------------------
namespace XG3D {
//------------------------------------------------------------
TexBuffer::TexBuffer(
    const ResTex& aResTex
    , ::XBase::IAllocator& aAllocator
    )
: mEXT()
, mResTex( aResTex )
, mAllocator( aAllocator )
{
    // チェック
    if ( !mResTex.isValid() )
    {
        XBASE_NOT_REACH_ASSERT();
        return;
    }

    // テクスチャ作成
    XG3D_GLCMD( glBindTexture( GL_TEXTURE_2D , mEXT.texId ) );
    XG3D_GLCMD( glTexParameteri( GL_TEXTURE_2D , GL_TEXTURE_MAG_FILTER , GL_LINEAR ) );
    XG3D_GLCMD( glTexParameteri( GL_TEXTURE_2D , GL_TEXTURE_MIN_FILTER , GL_LINEAR ) );
    XG3D_GLCMD( glTexImage2D(
        GL_TEXTURE_2D
        , 0
        , mResTex.format() == ResTexFormat_RGBA8 ? GL_RGBA : GL_RGB
        , mResTex.width()
        , mResTex.height()
        , 0
        , mResTex.format() == ResTexFormat_RGBA8 ? GL_RGBA : GL_RGB
        , GL_UNSIGNED_BYTE
        , mResTex.pixels()
        ));
    XG3D_GLCMD( glBindTexture( GL_TEXTURE_2D , 0 ) );
}

//------------------------------------------------------------
TexBuffer::~TexBuffer()
{
}

//------------------------------------------------------------
void TexBuffer::bind( TexSetting& aSetting )
{
    aSetting.ext_().texId = mEXT.texId;
}

//------------------------------------------------------------
TexBuffer_EXT::TexBuffer_EXT()
: texId()
{
    XG3D_GLCMD( glGenTextures( 1 , &texId ) );
}

//------------------------------------------------------------
TexBuffer_EXT::~TexBuffer_EXT()
{
    XG3D_GLCMD( glDeleteTextures( 1 , &texId ) );
}

//------------------------------------------------------------
}
//------------------------------------------------------------
// EOF
