﻿/**
 * @file
 * @brief ResTex型を記述する。
 * @author akino
 */
#if defined(XG3D_INCLUDED_RESTEX_HPP)
#else
#define XG3D_INCLUDED_RESTEX_HPP

//------------------------------------------------------------
#include <XG3D/BuiltInTypes.hpp>
#include <XG3D/ResTexFormat.hpp>
#include <XData/XData.hpp>

//------------------------------------------------------------
namespace XG3D {
    struct BinResTex;
}

//------------------------------------------------------------
namespace XG3D {
/// @addtogroup XG3D-Res
//@{
    /**
     * @brief テクスチャリソースのアクセサ。
     * @details
     * ただのアクセサなのでコピーして使って構いません。メモリのアロケーションも走りません。
     */
    class ResTex
    {
    public:
        /// @name コンストラクタとデストラクタ
        //@{
        /// 無効なオブジェクトを作成。
        ResTex();

        /**
         * @brief バイナリデータの先頭アドレスを指定して作成。
         * @details
         * 無効なバイナリデータの場合、isValid() == false になります。
         */
        ResTex( const_ptr_t aBin );
        //@}

        /// @name 正当性チェック
        //@{
        /// 有効なデータか。
        bool isValid()const;
        //@}

        /// @name プロパティの取得( isValid() なときしかアクセスできません)
        //@{
        u16 width()const;  ///< 横ピクセル数。
        u16 height()const; ///< 縦ピクセル数。
        ResTexFormat format()const;  ///< フォーマット。
        const byte_t* pixels()const; ///< ピクセルデータの先頭アドレス。
        //@}

    private:
        const_ptr_t mPtr;
        //------------------------------------------------------------
        bool checkInvalid()const;
        const BinResTex& bin()const;
        const_ptr_t ref( ::XData::Reference val )const;
    };
//@}
}
//------------------------------------------------------------
#endif
// EOF
