﻿/**
 * @file
 * @brief TexBuffer型を記述する。
 * @author akino
 */
#if defined(XG3D_INCLUDED_TEXBUFFER_HPP)
#else
#define XG3D_INCLUDED_TEXBUFFER_HPP

//------------------------------------------------------------
#include <XBase/IAllocator.hpp>
#include <XBase/NonCopyable.hpp>
#include <XG3D/BuiltInTypes.hpp>
#include <XG3D/Engine.hpp>
#include <XG3D/ResTex.hpp>

#if defined(XG3D_ENGINE_GLSERIES)
    #include <XG3D/TexBuffer_GL.hpp>
#endif

//------------------------------------------------------------
namespace XG3D {
    class TexSetting;
}

//------------------------------------------------------------
namespace XG3D {
/// @addtogroup XG3D-Util
//@{
    /**
     * @brief テクスチャバッファ。
     * @details
     * テクスチャのバッファの作成・保持をするクラスです。 @n
     * プログラムコードでテクスチャを生成・編集する用に作られています。 @n
     */
    class TexBuffer : public ::XBase::NonCopyable
    {
    public:
        /// @name コンストラクタとデストラクタ
        //@{
        /**
         * @brief ResTexをコピーして作成。
         * @param aResTex コピーされるResTex。
         * @param aAllocator テクスチャの一時領域を確保する際に使用するアロケータ。
         */
        TexBuffer( 
            const ResTex& aResTex
            , ::XBase::IAllocator& aAllocator = ::XBase::IAllocator::Default()
            );

        /// デストラクタ。
        ~TexBuffer();
        //@}
        
        /// @name 登録
        //@{
        void bind( TexSetting& aSetting ); ///< 指定の設定にこのテクスチャをバインドする。
        //@}

    private:
        TexBuffer_EXT mEXT;
        ResTex mResTex;
        ::XBase::IAllocator& mAllocator;
    };
//@}
}
//------------------------------------------------------------
#endif
// EOF
